from django.urls import path
from django.views.generic.base import RedirectView

from arkindex.dataimport.api import (
    AvailableRepositoriesList,
    CorpusWorkersActivity,
    CorpusWorkerVersionList,
    CorpusWorkflow,
    DataFileCreate,
    DataFileList,
    DataFileRetrieve,
    DataImportDetails,
    DataImportFromFiles,
    DataImportRetry,
    DataImportsList,
    GitRepositoryImportHook,
    ImportTranskribus,
    ListProcessElements,
    ProcessWorkersActivity,
    RepositoryList,
    RepositoryRetrieve,
    RevisionRetrieve,
    StartProcess,
    UpdateWorkerActivity,
    WorkerConfigurationList,
    WorkerList,
    WorkerRetrieve,
    WorkerRunDetails,
    WorkerRunList,
    WorkerVersionList,
    WorkerVersionRetrieve,
)
from arkindex.documents.api.elements import (
    AllowedMetaDataEdit,
    CorpusAllowedMetaData,
    CorpusElements,
    CorpusList,
    CorpusRetrieve,
    CorpusSelectionDestroy,
    ElementBulkCreate,
    ElementChildren,
    ElementMetadata,
    ElementMetadataBulk,
    ElementMove,
    ElementNeighbors,
    ElementParent,
    ElementParents,
    ElementRetrieve,
    ElementsCreate,
    ElementTranscriptions,
    ElementTypeCreate,
    ElementTypeUpdate,
    ManageSelection,
    MetadataEdit,
    WorkerResultsDestroy,
)
from arkindex.documents.api.entities import (
    CorpusEntities,
    CorpusRoles,
    ElementEntities,
    ElementLinks,
    EntityCreate,
    EntityDetails,
    EntityElements,
    EntityLinkCreate,
    TranscriptionEntities,
    TranscriptionEntityCreate,
)
from arkindex.documents.api.export import CorpusExportAPIView, DownloadExport
from arkindex.documents.api.iiif import ElementAnnotationList, FolderManifest
from arkindex.documents.api.ml import (
    ClassificationBulk,
    ClassificationCreate,
    ClassificationReject,
    ClassificationValidate,
    CorpusMLClassList,
    ElementTranscriptionsBulk,
    ManageClassificationsSelection,
    MLClassRetrieve,
    TranscriptionBulk,
    TranscriptionCreate,
    TranscriptionEdit,
)
from arkindex.documents.api.search import CorpusSearch
from arkindex.images.api import IIIFInformationCreate, IIIFURLCreate, ImageCreate, ImageElements, ImageRetrieve
from arkindex.project.openapi import OpenApiSchemaView
from arkindex.users.api import (
    CredentialsList,
    CredentialsRetrieve,
    GenericMembershipCreate,
    GenericMembershipsList,
    GroupDetails,
    GroupsCreate,
    JobList,
    JobRetrieve,
    MembershipDetails,
    OAuthCallback,
    OAuthRetry,
    OAuthSignIn,
    PasswordReset,
    PasswordResetConfirm,
    ProvidersList,
    UserCreate,
    UserEmailLogin,
    UserEmailVerification,
    UserMemberships,
    UserRetrieve,
    UserTranskribus,
)

api = [

    # Elements
    path('elements/<uuid:pk>/neighbors/', ElementNeighbors.as_view(), name='elements-neighbors'),
    path('elements/<uuid:pk>/parents/', ElementParents.as_view(), name='elements-parents'),
    path('elements/<uuid:pk>/children/', ElementChildren.as_view(), name='elements-children'),
    path('elements/selection/', ManageSelection.as_view(), name='elements-selection'),
    path('elements/create/', ElementsCreate.as_view(), name='elements-create'),
    path('elements/type/', ElementTypeCreate.as_view(), name='element-type-create'),
    path('elements/type/<uuid:pk>/', ElementTypeUpdate.as_view(), name='element-type'),
    path('element/<uuid:pk>/', ElementRetrieve.as_view(), name='element-retrieve'),
    path('element/<uuid:pk>/children/bulk/', ElementBulkCreate.as_view(), name='elements-bulk-create'),
    path('element/<uuid:pk>/metadata/', ElementMetadata.as_view(), name='element-metadata'),
    path('element/<uuid:pk>/metadata/bulk/', ElementMetadataBulk.as_view(), name='element-metadata-bulk'),
    path('element/<uuid:pk>/entities/', ElementEntities.as_view(), name='element-entities'),
    path('element/<uuid:pk>/links/', ElementLinks.as_view(), name='element-links'),
    path('element/<uuid:pk>/transcription/', TranscriptionCreate.as_view(), name='transcription-create'),
    path('element/<uuid:pk>/transcriptions/', ElementTranscriptions.as_view(), name='element-transcriptions'),
    path(
        'element/<uuid:pk>/transcriptions/bulk/',
        ElementTranscriptionsBulk.as_view(),
        name='element-transcriptions-bulk'
    ),
    path('element/<uuid:child>/parent/<uuid:parent>/', ElementParent.as_view(), name='element-parent'),
    path('element/move/', ElementMove.as_view(), name='move-element'),

    # Corpora
    path('corpus/', CorpusList.as_view(), name='corpus'),
    path('corpus/<uuid:pk>/', CorpusRetrieve.as_view(), name='corpus-retrieve'),
    # `corpus` and not `pk` so that OpenAPI uses a `corpus` parameter
    path('corpus/<uuid:corpus>/elements/', CorpusElements.as_view(), name='corpus-elements'),
    path('corpus/<uuid:pk>/classes/', CorpusMLClassList.as_view(), name='corpus-classes'),
    path('corpus/<uuid:corpus>/classes/<uuid:mlclass>/', MLClassRetrieve.as_view(), name='ml-class-retrieve'),
    path('corpus/<uuid:pk>/entities/', CorpusEntities.as_view(), name='corpus-entities'),
    path('corpus/<uuid:pk>/roles/', CorpusRoles.as_view(), name='corpus-roles'),
    path('corpus/<uuid:pk>/allowed-metadata/', CorpusAllowedMetaData.as_view(), name='corpus-allowed-metadata'),
    path('corpus/<uuid:corpus>/allowed-metadata/<uuid:pk>/', AllowedMetaDataEdit.as_view(), name='allowed-metadata-edit'),
    path('corpus/<uuid:pk>/versions/', CorpusWorkerVersionList.as_view(), name='corpus-versions'),
    path('corpus/<uuid:pk>/selection/', CorpusSelectionDestroy.as_view(), name='corpus-delete-selection'),
    path('corpus/<uuid:pk>/search/', CorpusSearch.as_view(), name='corpus-search'),
    path('corpus/<uuid:pk>/export/', CorpusExportAPIView.as_view(), name='corpus-export'),
    path('corpus/<uuid:corpus>/workerversion/<uuid:version>/results/', WorkerResultsDestroy.as_view(), name='worker-delete-results'),
    path('corpus/<uuid:corpus>/workers-activity/', CorpusWorkersActivity.as_view(), name='corpus-workers-activity'),
    path('export/<uuid:pk>/', DownloadExport.as_view(), name='download-export'),

    # Moderation
    path('classifications/', ClassificationCreate.as_view(), name='classification-create'),
    path('classifications/<uuid:pk>/validate/', ClassificationValidate.as_view(), name='classification-validate'),
    path('classifications/<uuid:pk>/reject/', ClassificationReject.as_view(), name='classification-reject'),
    path('classifications/selection/', ManageClassificationsSelection.as_view(), name='classification-selection'),

    # Manifests
    path('iiif/<uuid:pk>/manifest/', FolderManifest.as_view(), name='folder-manifest'),
    path('iiif/<uuid:pk>/list/transcriptions/', ElementAnnotationList.as_view(), name='element-annotation-list'),

    # Placeholder URLs for IIIF IDs
    path(
        'iiif/<uuid:pk>/sequence/',
        RedirectView.as_view(pattern_name='api:folder-manifest', permanent=False),
        name='iiif-sequence',
    ),
    path(
        'iiif/<uuid:pk>/canvas/',
        RedirectView.as_view(pattern_name='api:element-annotation-list', permanent=False),
        name='iiif-canvas',
    ),
    path(
        'iiif/<uuid:page_pk>/annotation/<uuid:transcription_pk>/',
        RedirectView.as_view(pattern_name='api:element-annotation-list', permanent=False),
        name='transcription-annotation',
    ),

    # Ingest transcriptions
    path('transcription/<uuid:pk>/', TranscriptionEdit.as_view(), name='transcription-edit'),
    path('transcription/bulk/', TranscriptionBulk.as_view(), name='transcription-bulk'),
    path('classification/bulk/', ClassificationBulk.as_view(), name='classification-bulk'),

    # Ingest and retrieve entities
    path('entity/', EntityCreate.as_view(), name='entity-create'),
    path('entity/link/', EntityLinkCreate.as_view(), name='entity-link-create'),
    path('entity/<uuid:pk>/', EntityDetails.as_view(), name='entity-details'),
    path('entity/<uuid:pk>/elements/', EntityElements.as_view(), name='entity-elements'),
    path('transcription/<uuid:pk>/entity/', TranscriptionEntityCreate.as_view(), name='transcription-entity-create'),
    path('transcription/<uuid:pk>/entities/', TranscriptionEntities.as_view(), name='transcription-entities'),

    # Ingest metadata
    path('metadata/<uuid:pk>/', MetadataEdit.as_view(), name='metadata-edit'),

    # Git import workflows
    path('imports/repos/', RepositoryList.as_view(), name='repository-list'),
    path('imports/repos/<uuid:pk>/', RepositoryRetrieve.as_view(), name='repository-retrieve'),
    path('imports/repos/search/<uuid:pk>/', AvailableRepositoriesList.as_view(), name='available-repositories'),
    path('imports/hook/<uuid:pk>/', GitRepositoryImportHook.as_view(), name='import-hook'),
    path('imports/revisions/<uuid:pk>/', RevisionRetrieve.as_view(), name='revision-retrieve'),

    # Workers
    path('workers/', WorkerList.as_view(), name='workers-list'),
    path('workers/<uuid:pk>/', WorkerRetrieve.as_view(), name='worker-retrieve'),
    path('workers/<uuid:pk>/configurations/', WorkerConfigurationList.as_view(), name='worker-configurations'),
    path('workers/<uuid:pk>/versions/', WorkerVersionList.as_view(), name='worker-versions'),
    path('workers/versions/<uuid:pk>/', WorkerVersionRetrieve.as_view(), name='version-retrieve'),
    path('workers/versions/<uuid:pk>/activity/', UpdateWorkerActivity.as_view(), name='update-worker-activity'),

    # Import workflows
    path('imports/', DataImportsList.as_view(), name='import-list'),
    path('imports/fromfiles/', DataImportFromFiles.as_view(), name='import-from-files'),
    path('imports/transkribus/', ImportTranskribus.as_view(), name='import-transkribus'),
    path('imports/<uuid:pk>/', DataImportDetails.as_view(), name='import-details'),
    path('imports/<uuid:pk>/retry/', DataImportRetry.as_view(), name='import-retry'),
    path('imports/<uuid:pk>/start/', StartProcess.as_view(), name='process-start'),
    path('imports/files/<uuid:pk>/', DataFileList.as_view(), name='file-list'),
    path('imports/files/create/', DataFileCreate.as_view(), name='file-create'),
    path('imports/file/<uuid:pk>/', DataFileRetrieve.as_view(), name='file-retrieve'),
    path('imports/corpus/', CorpusWorkflow.as_view(), name='corpus-workflow'),
    path('imports/<uuid:pk>/workers/', WorkerRunList.as_view(), name='worker-run-list'),
    path('imports/workers/<uuid:pk>/', WorkerRunDetails.as_view(), name='worker-run-details'),
    path('process/<uuid:pk>/elements/', ListProcessElements.as_view(), name='process-elements-list'),
    path('process/<uuid:pk>/workers-activity/', ProcessWorkersActivity.as_view(), name='process-workers-activity'),

    # Image management
    path('image/', ImageCreate.as_view(), name='image-create'),
    path('image/iiif/url/', IIIFURLCreate.as_view(), name='iiif-url-create'),
    path('image/iiif/info/', IIIFInformationCreate.as_view(), name='iiif-info-create'),
    path('image/<uuid:pk>/', ImageRetrieve.as_view(), name='image-retrieve'),
    path('image/<uuid:pk>/elements/', ImageElements.as_view(), name='image-elements'),

    # Manage OAuth integrations
    path('oauth/providers/', ProvidersList.as_view(), name='providers-list'),
    path('oauth/providers/<provider>/signin/', OAuthSignIn.as_view(), name='oauth-signin'),
    path('oauth/providers/<provider>/callback/', OAuthCallback.as_view(), name='oauth-callback'),
    path('oauth/credentials/', CredentialsList.as_view(), name='credentials-list'),
    path('oauth/credentials/<uuid:pk>/', CredentialsRetrieve.as_view(), name='credentials-retrieve'),
    path('oauth/credentials/<uuid:pk>/retry/', OAuthRetry.as_view(), name='oauth-retry'),

    # Authentication
    path('user/', UserRetrieve.as_view(), name='user-retrieve'),
    path('user/new/', UserCreate.as_view(), name='user-new'),
    path('user/login/', UserEmailLogin.as_view(), name='user-login'),
    path('user/token/', UserEmailVerification.as_view(), name='user-token'),
    path('user/password-reset/', PasswordReset.as_view(), name='password-reset'),
    path('user/password-reset/confirm/', PasswordResetConfirm.as_view(), name='password-reset-confirm'),
    path('user/transkribus/', UserTranskribus.as_view(), name='user-transkribus'),

    # Rights management
    path('groups/', GroupsCreate.as_view(), name='groups-create'),
    path('group/<uuid:pk>/', GroupDetails.as_view(), name='group-details'),
    path('user/memberships/', UserMemberships.as_view(), name='user-memberships'),
    path('membership/<uuid:pk>/', MembershipDetails.as_view(), name='membership-details'),
    path('memberships/', GenericMembershipsList.as_view(), name='memberships-list'),
    path('memberships/create/', GenericMembershipCreate.as_view(), name='membership-create'),

    # Asynchronous jobs
    path('jobs/', JobList.as_view(), name='jobs-list'),
    path('jobs/<uuid:pk>/', JobRetrieve.as_view(), name='jobs-retrieve'),

    # OpenAPI Schema
    path('openapi/', OpenApiSchemaView.as_view(), name='openapi-schema'),
]
